package org.apache.cordova.mediacapture.custom.activity;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.MediaRecorder;
import android.net.Uri;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Environment;
import android.os.SystemClock;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import __ANDROID_PACKAGE__.R;

import org.apache.cordova.mediacapture.custom.cameraview.CameraListener;
import org.apache.cordova.mediacapture.custom.cameraview.CameraOptions;
import org.apache.cordova.mediacapture.custom.cameraview.CameraView;
import org.apache.cordova.mediacapture.custom.options.Hdr;
import org.apache.cordova.mediacapture.custom.options.SessionType;
import org.apache.cordova.mediacapture.custom.options.VideoQuality;
import org.apache.cordova.mediacapture.custom.utils.CameraLogger;

import java.io.File;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Locale;

/**
 * Created by oliver on 17-11-6.
 */

public class CameraActivity extends Activity implements View.OnClickListener {
    private static final String TAG = "CameraActivity";
    private CameraView camera;
    private LinearLayout llTime;
    private ImageView ivCapture;
    private TextView tvRestTime;
    private File saveDir;
    public static final String EXTRA_VIDEO_OPTION = "param1";
    private VideoOption videoOption;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED,
                WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED);
        setContentView(R.layout.media_activity_camera);
        CameraLogger.setLogLevel(CameraLogger.LEVEL_ERROR);
        camera = (CameraView) findViewById(R.id.camera);
        llTime = (LinearLayout) findViewById(R.id.ll_rest_time);
        ivCapture = (ImageView) findViewById(R.id.iv_capture);
        tvRestTime = (TextView) findViewById(R.id.tv_time);
        ivCapture.setOnClickListener(this);
        if(getIntent().getSerializableExtra(EXTRA_VIDEO_OPTION)!=null){
            videoOption = (VideoOption) getIntent().getSerializableExtra(EXTRA_VIDEO_OPTION);
        }else {
            videoOption = new VideoOption(-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,10000,"MAX_480P"); //默认10s
        }
        saveDir = new File(getCacheDir(),"capture_video");
        saveDir.mkdirs();
        format = new SimpleDateFormat("mm:ss", Locale.getDefault());
        camera.setVideoOption(videoOption);
        camera.addCameraListener(new CameraListener() {
            @Override
            public void onCameraOpened(CameraOptions options) { onOpened(options); }
            @Override
            public void onPictureTaken(byte[] jpeg) { onPicture(jpeg); }
            @Override
            public void onVideoTaken(File video) { onVideo(video); }
        });
    }
    long startTime;
    private void onVideo(File video) {
        long length = video.length();
        Intent intent = new Intent();
        Uri uri = Uri.fromFile(video);
        intent.setData(uri);
        Log.d(TAG, "onVideo: size = "+(length/1024f/1024f)+"mb"+" path = "+uri.toString());
        setResult(RESULT_OK,intent);
        finish();
    }

    @Override
    public void onBackPressed() {
        setResult(RESULT_CANCELED);
        super.onBackPressed();
    }

    private void onPicture(byte[] jpeg) {

    }

    private void onOpened(CameraOptions options) {

    }
    private CountDownTimer timer;
    private File mVideoFile;
    private void captureVideo(){
        String ext = ".mp4";
        if(videoOption.fileFormat == MediaRecorder.OutputFormat.THREE_GPP){
            ext = ".3gp";
        }
        mVideoFile = new File(saveDir,System.currentTimeMillis()+ext);
        if (camera.getSessionType() != SessionType.VIDEO) {
            Toast.makeText(this, "当前是拍摄模式", Toast.LENGTH_SHORT).show();
            return;
        }
        camera.setVideoQuality(VideoQuality.valueOf(videoOption.getMaxQuality()));
        timer = new CountDownTimer(videoOption.getDuration()+1000,100){
            @Override
            public void onFinish() {
                updateTime();
                stopVideoCapture();
            }

            @Override
            public void onTick(long l) {
                updateTime();

                if(videoOption.maxFileLength!=-1){
                    float v = mVideoFile.length() / 1024f / 1024f;
                    if(v >= videoOption.maxFileLength-0.5f){
                        timer.cancel();
                        timer.onFinish();
                    }
                }
            }
        };
        startTime = System.currentTimeMillis();
        timer.start();
        llTime.setVisibility(View.VISIBLE);
        ivCapture.setTag(true);
        ivCapture.setImageResource(R.drawable.media_video_record_stop);
        camera.startCapturingVideo(mVideoFile);
    }

    private SimpleDateFormat format;

    private void updateTime(){
        long span = System.currentTimeMillis() - startTime;
        String formatStr = format.format(span);
        tvRestTime.setText(formatStr);
    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        if(id == R.id.iv_capture){
            if(view.getTag() == null){
                captureVideo();
            }else {
                if(timer!=null){
                    timer.cancel();
                    timer.onFinish();
                }
            }
        }
    }

    private void stopVideoCapture(){
        llTime.setVisibility(View.GONE);
        camera.stopCapturingVideo();
        ivCapture.setTag(null);
        ivCapture.setImageResource(R.drawable.media_video_record_start);
        tvRestTime.setText("00:00");
    }

    @Override
    protected void onResume() {
        super.onResume();
        camera.start();
    }

    @Override
    protected void onPause() {
        super.onPause();
        camera.stop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        camera.destroy();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        boolean valid = true;
        for (int grantResult : grantResults) {
            valid = valid && grantResult == PackageManager.PERMISSION_GRANTED;
        }
        if (valid && !camera.isStarted()) {
            camera.start();
        }
    }


    public static class VideoOption implements Serializable {
        public int audioBitRate;
        public int audioChannels;
        public int audioCodec;
        public int audioSampleRate;
        public int fileFormat;
        public int videoBitRate;
        public int videoCodec;
        public int videoFrameHeight;
        public int videoFrameRate;
        public int videoFrameWidth;
        int duration;
        public String maxQuality;
        public int maxFileLength = -1;

        public VideoOption(){}

        public VideoOption(int audioBitRate, int audioChannels, int audioCodec, int audioSampleRate, int fileFormat,
                           int videoBitRate, int videoCodec, int videoFrameHeight, int videoFrameRate, int videoFrameWidth,
                           int duration, String maxQuality) {
            this.audioBitRate = audioBitRate;
            this.audioChannels = audioChannels;
            this.audioCodec = audioCodec;
            this.audioSampleRate = audioSampleRate;
            this.fileFormat = fileFormat;
            this.videoBitRate = videoBitRate;
            this.videoCodec = videoCodec;
            this.videoFrameHeight = videoFrameHeight;
            this.videoFrameRate = videoFrameRate;
            this.videoFrameWidth = videoFrameWidth;
            this.duration = duration;
            this.maxQuality = maxQuality;
        }

        public int getMaxFileLength() {
            return maxFileLength;
        }

        public void setMaxFileLength(int maxFileLength) {
            this.maxFileLength = maxFileLength;
        }

        public int getAudioBitRate() {
            return audioBitRate;
        }

        public void setAudioBitRate(int audioBitRate) {
            this.audioBitRate = audioBitRate;
        }

        public int getAudioChannels() {
            return audioChannels;
        }

        public void setAudioChannels(int audioChannels) {
            this.audioChannels = audioChannels;
        }

        public int getAudioCodec() {
            return audioCodec;
        }

        public void setAudioCodec(int audioCodec) {
            this.audioCodec = audioCodec;
        }

        public int getAudioSampleRate() {
            return audioSampleRate;
        }

        public void setAudioSampleRate(int audioSampleRate) {
            this.audioSampleRate = audioSampleRate;
        }

        public int getFileFormat() {
            return fileFormat;
        }

        public void setFileFormat(int fileFormat) {
            this.fileFormat = fileFormat;
        }

        public int getVideoBitRate() {
            return videoBitRate;
        }

        public void setVideoBitRate(int videoBitRate) {
            this.videoBitRate = videoBitRate;
        }

        public int getVideoCodec() {
            return videoCodec;
        }

        public void setVideoCodec(int videoCodec) {
            this.videoCodec = videoCodec;
        }

        public int getVideoFrameHeight() {
            return videoFrameHeight;
        }

        public void setVideoFrameHeight(int videoFrameHeight) {
            this.videoFrameHeight = videoFrameHeight;
        }

        public int getVideoFrameRate() {
            return videoFrameRate;
        }

        public void setVideoFrameRate(int videoFrameRate) {
            this.videoFrameRate = videoFrameRate;
        }

        public int getVideoFrameWidth() {
            return videoFrameWidth;
        }

        public void setVideoFrameWidth(int videoFrameWidth) {
            this.videoFrameWidth = videoFrameWidth;
        }

        public int getDuration() {
            return duration;
        }

        public void setDuration(int duration) {
            this.duration = duration;
        }

        public String getMaxQuality() {
            return maxQuality;
        }

        public void setMaxQuality(String maxQuality) {
            this.maxQuality = maxQuality;
        }

        @Override
        public String toString() {
            return "VideoOption{" +
                    "audioBitRate=" + audioBitRate +
                    ", audioChannels=" + audioChannels +
                    ", audioCodec=" + audioCodec +
                    ", audioSampleRate=" + audioSampleRate +
                    ", fileFormat=" + fileFormat +
                    ", videoBitRate=" + videoBitRate +
                    ", videoCodec=" + videoCodec +
                    ", videoFrameHeight=" + videoFrameHeight +
                    ", videoFrameRate=" + videoFrameRate +
                    ", videoFrameWidth=" + videoFrameWidth +
                    ", duration=" + duration +
                    ", maxQuality='" + maxQuality + '\'' +
                    '}';
        }
    }

}
